#include "Engine.hpp"
#include "bindings.hpp"

int Engine::maxHeight = 1;
int Engine::maxWidth = 1;
int	Engine::ammo = 20;

void Engine::start() {
    this->frame = initscr();    // renvoie l'addresse de la fenetre creee
    if(!has_colors()) {
        this->crash("Cannot play if colors are disabled !");
        return ;
    } else {
        start_color();              // rend l'utilisation des couleurs possible
    }
	noecho();                       // desactive l'affichage de caractere quand on appuie sur les touches
	curs_set(0);                    // cache le curseur du terminal
    keypad(this->frame, TRUE);       // rend possible la detection de pression sur les touches fleches
    nodelay(this->frame, TRUE);      // rend l'input (avec getch) non bloquant (asynchrome en quelque sorte)

    // recupere les dimensions de la fenetre (getmaxyx est une macro qui set les deux derniers parametres)
    getmaxyx(this->frame, Engine::maxHeight, Engine::maxWidth);
    if (Engine::maxHeight <= 20 || Engine::maxWidth <= 100) {
        this->crash("The terminal is way too small ! Please enlarge the window.");
    }

    // donne des bordures au spawner d etoiles
    this->pilot.setPosition(Engine::maxHeight / 2, ((Engine::maxWidth / 2) / 2) * 2);
    init_color(COLOR_RED, 250, 250, 250);
    init_pair(1, COLOR_RED, COLOR_BLACK);
    init_pair(2, COLOR_YELLOW, COLOR_BLACK);
}

// Game loop
void Engine::launch() {
    while (42) {
        if (this->gameOver) {
            this->printGameOver();
            refresh();
            usleep(1000000);
            return;
        }

        clear();                                    // efface tout l'ecran
        this->stars.updateObjects();                // fait bouger les etoiles (.) en background
        this->enemies.updateObjects();              // fait bouger les ennemis
		this->enemies.maybeShoot(this->e_rockets);
		this->asteroids.updateObjects();
		this->e_rockets.updateObjects(maxHeight);
        this->manageCollision();                    // detruit les ennemis touchés
        this->pilot.getRockets().updateObjects();   // fait bouger les missiles du pilote
        pilot.move();                               // fait bouger le pilote
        this->manageCollision();                    // detruit les ennemis touchés

        this->printGame();                          // met le jeu dans le buffer ncurses
        refresh();                                  // rafraichit la fenetre du terminal

        this->keyGesture(wgetch(this->frame));      // detecte les touches
    }
}

void Engine::manageCollision(void) {

	if (this->asteroids.checkCollision(&(this->pilot)) == true)
		this->gameOver = true;

	if (this->enemies.checkCollision(&(this->pilot), &(this->score)) == true)
		this->gameOver = true;

	if (this->e_rockets.checkCollision(&(this->pilot)) == true)
		this->gameOver = true;
}

void Engine::printGame() {
    // affiche le pilote
    attron(A_UNDERLINE);
    attron(A_BOLD);
    attron(COLOR_PAIR(1));
    mvaddch(this->pilot.getPosition().y, this->pilot.getPosition().x, this->pilot.getShape());
    attroff(COLOR_PAIR(1));
    attroff(A_BOLD);
    attroff(A_UNDERLINE);

    // affiche toutes les etoiles ('.') qui passent en arriere plan
    for (int i = 0; i < this->stars.getSize(); i++) {
        AObject *star = this->stars.get(i);
        if (star->getEnabled()) {
            int x = star->getPosition().x;
            int y = star->getPosition().y;
            mvaddch(y, x, star->getShape());
        }
    }

    // affiche tous les ennemis ('O')
    for (int i = 0; i < this->enemies.getSize(); i++) {
        AObject *enemy = this->enemies.get(i);
        if (enemy->getEnabled()) {
            int x = enemy->getPosition().x;
            int y = enemy->getPosition().y;
            mvaddch(y, x, enemy->getShape());
        }
    }

    // affiche tous les missiles du pilote ('|')
    for (int i = 0; i < this->pilot.getRockets().getSize(); i++) {
        AObject *rocket = this->pilot.getRockets().get(i);
        if (rocket->getEnabled()) {
            int x = rocket->getPosition().x;
            int y = rocket->getPosition().y;
            mvaddch(y, x, rocket->getShape());
        }
    }

	for (int i = 0; i < this->asteroids.getSize(); i++) {
        Asteroid *asteroid = (Asteroid*)this->asteroids.get(i);
        if (asteroid->getEnabled()) {
            int x = asteroid->getPosition().x;
            int y = asteroid->getPosition().y;
            mvaddch(y, x, asteroid->getShape(0));
			mvaddch(y, x+1, asteroid->getShape(1));
			mvaddch(y, x+2, asteroid->getShape(2));
			mvaddch(y, x+3, asteroid->getShape(3));
        }
    }

	for (int i = 0; i < this->e_rockets.getSize(); i++) {
        AObject *e_rocket = this->e_rockets.get(i);
        if (e_rocket->getEnabled()) {
            int x = e_rocket->getPosition().x;
            int y = e_rocket->getPosition().y;
            mvaddch(y, x, e_rocket->getShape());
        }
    }

    mvprintw(1, 2, "Score: %d", this->score);
	mvprintw(2, 2, "Ammo: %d", this->ammo);
}


//////////////////////////////////////////////////////////
///
///            THINGS THAT DOESNT MATTER
///
//////////////////////////////////////////////////////////

void Engine::keyGesture(char key) {
    switch (key) {
        case KEY_ARROW_LEFT:
        case KEY_ARROW_RIGHT:
        case KEY_ARROW_UP:
        case KEY_ARROW_DOWN:
            this->pilot.setDirection(key);
            key = -1;
            break;
        case KEY_SPACE:
            this->pilot.shoot();
            break;
        case KEY_ECHAP:
            this->finish();
            exit(0);
            break;
        default:
            break;

    }
}

Engine::Engine(void) {
    this->gameOver = false;
    this->score = 0;
    this->start();
}

Engine::Engine(const Engine & src) {
    *this = src;
    this->start();
}

void Engine::printGameOver() {
    mvprintw((Engine::maxHeight / 2    ) - 2, (Engine::maxWidth - 55) / 2, "   ___                                                 \n");
    mvprintw((Engine::maxHeight / 2 + 1) - 2, (Engine::maxWidth - 55) / 2, "  / _ \\ __ _  _ __ ___    ___    ___ __   __ ___  _ __ \n");
    mvprintw((Engine::maxHeight / 2 + 2) - 2, (Engine::maxWidth - 55) / 2, " / /_\\// _` || '_ ` _ \\  / _ \\  / _ \\\\ \\ / // _ \\| '__|\n");
    mvprintw((Engine::maxHeight / 2 + 3) - 2, (Engine::maxWidth - 55) / 2, "/ /_\\\\| (_| || | | | | ||  __/ | (_) |\\ V /|  __/| |   \n");
    mvprintw((Engine::maxHeight / 2 + 4) - 2, (Engine::maxWidth - 55) / 2, "\\____/ \\__,_||_| |_| |_| \\___|  \\___/  \\_/  \\___||_|   \n");
}

void Engine::finish() {
    clear();
    refresh();
    endwin();
}

void Engine::crash(std::string const stopMessage) {
    clear();
    refresh();
    endwin();
    std::cerr << stopMessage << std::endl;
    exit(-1);
}

Engine::~Engine() {
}

Engine & Engine::operator=(Engine const &copy) {
    if (this != &copy){
        this->pilot = copy.pilot;
        this->frame = copy.frame;
        this->stars = copy.stars;
        this->enemies = copy.enemies;
        this->gameOver = copy.gameOver;
        this->score = copy.score;
    }
    this->start();
    return *this;
}
